#
# Aura Engine (https://gitlab.servus.at/aura/engine)
#
# Copyright (C) 2017-2020 - The Aura Engine Team.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import json

from modules.cli.redis.channels import RedisChannel
from modules.base.utils import TerminalColors
from modules.cli.redis.adapter import ClientRedisAdapter, ServerRedisAdapter
from modules.base.models import AuraDatabaseModel


class Padavan:
    args = None
    config = None

    ss = None
    zmqclient = None
    redisclient = None
    stringreply = ""

    # ------------------------------------------------------------------------------------------ #
    def __init__(self, args, config):
        self.args = args
        self.config = config

    # ------------------------------------------------------------------------------------------ #
    def meditate(self):
        if self.args.fetch_new_programme:
            self.fetch_new_programme()

        elif self.args.mixer_channels_selected:
            self.mixer_channels_selected()

        elif self.args.mixer_status:
            self.mixer_status()

        elif self.args.get_act_programme:
            self.get_act_programme()

        elif self.args.get_status:
            self.get_status()

        elif self.args.get_connection_status:
            self.get_connection_status()

        elif self.args.shutdown:
            self.shutdown()

        elif self.args.redis_message:
            self.redis_message(self.args.redis_message[0], self.args.redis_message[1])

        elif self.args.select_mixer != -1:
            self.select_mixer(self.args.select_mixer)

        elif self.args.deselect_mixer != -1:
            self.select_mixer(self.args.deselect_mixer, False)

        elif self.args.set_volume:
            self.set_volume(self.args.set_volume[0], self.args.set_volume[1])

        elif self.args.print_message_queue:
            self.print_message_queue()

        elif self.args.get_file_for:
            self.get_next_file(self.args.get_file_for)

        elif self.args.set_file_for:
            self.set_next_file(self.args.set_file_for[0], self.args.set_file_for[1])

        elif self.args.now_playing:
            print("")

        elif self.args.init_player:
            self.init_player()

        elif self.args.on_play:
            self.on_play(self.args.on_play)

        elif self.args.recreatedb:
            self.recreatedb()

        # else:
        #    raise Exception("")

    # init liquid => faster exec time, when loading at runtime just what is needed
    # ------------------------------------------------------------------------------------------ #
    def init_liquidsoap_communication(self):
        # import
        from modules.core.engine import SoundSystem

        # init liquidsoap communication
        self.ss = SoundSystem(self.config)
        # enable connection
        self.ss.enable_transaction()

    # ------------------------------------------------------------------------------------------ #
    def destroy_liquidsoap_communication(self):
        # enable connection
        self.ss.disable_transaction()

    # ------------------------------------------------------------------------------------------ #
    def init_redis_communication(self, with_server=False):
        self.redisclient = ClientRedisAdapter(self.config)

        if with_server:
            self.redisserver = ServerRedisAdapter(self.config)

    # ------------------------------------------------------------------------------------------ #
    def send_redis(self, channel, message):
        self.init_redis_communication()
        self.redisclient.publish(channel, message)

    # ------------------------------------------------------------------------------------------ #
    def send_and_wait_redis(self, channel, message, reply_channel):
        self.init_redis_communication(True)
        self.redisclient.publish(channel, message)
        return self.redisserver.listen_for_one_message(reply_channel.value)

    # ------------------------------------------------------------------------------------------ #
    def shutdown(self):
        self.send_redis("aura", "shutdown")
        self.stringreply = "Shutdown message sent!"

    # ------------------------------------------------------------------------------------------ #
    def fetch_new_programme(self):
        json_reply = self.send_and_wait_redis("aura", "fetch_new_programme", RedisChannel.FNP_REPLY)
        if json_reply != "":
            actprogramme = json.loads(json_reply)
            self.print_programme(actprogramme)
        else:
            print("No programme fetched")

    # ------------------------------------------------------------------------------------------ #
    def get_act_programme(self):
        json_reply = self.send_and_wait_redis("aura", "get_act_programme", RedisChannel.GAP_REPLY)
        actprogramme = json.loads(json_reply)
        self.print_programme(actprogramme)


    def get_status(self):
        """
        Retrieves the Engine's status information.
        """
        json_reply = self.send_and_wait_redis("aura", "get_status", RedisChannel.GS_REPLY)
        # status = json.loads(json_reply)
        self.stringreply = json_reply
        

    # ------------------------------------------------------------------------------------------ #
    def get_connection_status(self):
        json_reply = self.send_and_wait_redis("aura", "get_connection_status", RedisChannel.GCS_REPLY)
        connection_status = json.loads(json_reply)
        self.print_connection_status(connection_status)

    # ------------------------------------------------------------------------------------------ #
    def print_programme(self, programme):
        cnt = 1
        for show in programme:
            for entry in show["playlist"]:
                self.stringreply += str(cnt) + \
                    " --- schedule id #" + str(show["schedule_id"]) + "." + str(entry["entry_num"]) + \
                    " - show: " + show["show_name"] + \
                    " - starts @ " + entry["entry_start"] + \
                    " - plays " + str(entry["source"]) + "\n"
                cnt = cnt + 1

    # ------------------------------------------------------------------------------------------ #
    def print_connection_status(self, connection_status):

        if connection_status["pv"]:
            self.stringreply = "Connection to pv:    " + TerminalColors.GREEN.value + " " + str(connection_status["pv"]) + TerminalColors.ENDC.value
        else:
            self.stringreply = "Connection to pv:    " + TerminalColors.RED.value + " " + str(connection_status["pv"]) + TerminalColors.ENDC.value

        if connection_status["db"]:
            self.stringreply += "\nConnection to db:    " + TerminalColors.GREEN.value + " " + str(connection_status["db"]) + TerminalColors.ENDC.value
        else:
            self.stringreply += "\nConnection to db:    " + TerminalColors.RED.value + " " + str(connection_status["db"]) + TerminalColors.ENDC.value

        if connection_status["lqs"]:
            self.stringreply += "\nConnection to lqs:   " + TerminalColors.GREEN.value + " " + str(connection_status["lqs"]) + TerminalColors.ENDC.value
        else:
            self.stringreply += "\nConnection to lqs:   " + TerminalColors.RED.value + " " + str(connection_status["lqs"]) + TerminalColors.ENDC.value

        if connection_status["lqsr"]:
            self.stringreply += "\nConnection to lqsr:  " + TerminalColors.GREEN.value + " " + str(connection_status["lqsr"]) + TerminalColors.ENDC.value
        else:
            self.stringreply += "\nConnection to lqsr:  " + TerminalColors.RED.value + " " + str(connection_status["lqsr"]) + TerminalColors.ENDC.value

        if connection_status["tank"]:
            self.stringreply += "\nConnection to tank:  " + TerminalColors.GREEN.value + " " + str(connection_status["tank"]) + TerminalColors.ENDC.value
        else:
            self.stringreply += "\nConnection to tank:  " + TerminalColors.RED.value + " " + str(connection_status["tank"]) + TerminalColors.ENDC.value

        if connection_status["redis"]:
            self.stringreply += "\nConnection to redis: " + TerminalColors.GREEN.value + " " + str(connection_status["redis"]) + TerminalColors.ENDC.value
        else:
            self.stringreply += "\nConnection to redis: " + TerminalColors.RED.value + " " + str(connection_status["redis"]) + TerminalColors.ENDC.value

    # ------------------------------------------------------------------------------------------ #
    
    
    def init_player(self):
        """
        Initializes the player on Liquidsaop startup.
        """
        self.stringreply = self.send_and_wait_redis("aura", "init_player", RedisChannel.IP_REPLY)



    def on_play(self, info):
        """
        Event handler to be called when some entry started playing.
        """

        self.stringreply = self.send_and_wait_redis("aura", "on_play " + info, RedisChannel.GNF_REPLY)

    # ------------------------------------------------------------------------------------------ #
    def recreatedb(self):
        print("YOU WILL GET PROBLEMS DUE TO DATABASE BLOCKING IF aura.py IS RUNNING! NO CHECKS IMPLEMENTED SO FAR!")
        x = AuraDatabaseModel()
        x.recreate_db()
        self.stringreply = "Database recreated!"

    # ------------------------------------------------------------------------------------------ #
    def redis_message(self, channel, message):
        self.send_redis(channel, message)
        self.stringreply = "Message '"+message+"' sent to channel '"+channel+"'"

    # ------------------------------------------------------------------------------------------ #
    def print_message_queue(self):
        self.stringreply = self.send_and_wait_redis("aura", "print_message_queue", RedisChannel.PMQ_REPLY)

    # LIQUIDSOAP #

    # ------------------------------------------------------------------------------------------ #
    def select_mixer(self, mixername, activate=True):
        # init lqs
        self.init_liquidsoap_communication()

        # select mixer and return the feedback
        self.stringreply = self.ss.channel_activate(mixername, activate)

        # disable connection
        self.destroy_liquidsoap_communication()

    # ------------------------------------------------------------------------------------------ #
    def set_volume(self, mixernumber, volume):
        # init lqs and enable comm
        self.init_liquidsoap_communication()
        self.stringreply = self.ss.channel_volume(mixernumber, volume)
        # disable connection
        self.destroy_liquidsoap_communication()

    # ------------------------------------------------------------------------------------------ #
    def mixer_channels_selected(self):
        self.init_liquidsoap_communication()
        am = self.ss.mixer_channels_selected()

        if len(am) == 0:
            self.destroy_liquidsoap_communication()
            raise Exception("Guru recognized a problem: No active source!!!")

        self.stringreply = str(am)

        # disable connection
        self.destroy_liquidsoap_communication()

    # ------------------------------------------------------------------------------------------ #
    def mixer_status(self):
        self.init_liquidsoap_communication()

        status = self.ss.mixer_status()

        for k, v in status.items():
            self.stringreply += "source: " + k + "\t status: " + v + "\n"

        # disable connection
        self.destroy_liquidsoap_communication()

    # REDIS #

    # ------------------------------------------------------------------------------------------ #
    def get_next_file(self, type):
#        redis = RedisMessenger()
#        next_file = redis.get_next_file_for(type)
#        if next_file == "":
#            next_file = "/var/audio/blank.flac"
#        self.stringreply = next_file

        #self.send_redis("aura", "set_next_file " + type)

        next_file = self.send_and_wait_redis("aura", "get_next_file " + type, RedisChannel.GNF_REPLY)
        self.stringreply = next_file

    # ------------------------------------------------------------------------------------------ #
    def set_next_file(self, type, file):
        #from modules.cli.redis.messenger import RedisMessenger
        #redis = RedisMessenger()

        #redis.set_next_file_for(type, file)
        self.send_redis("aura", "set_next_file " + type + " " + file)
        self.stringreply = "Set "+file+" for fallback '"+type+"'"
